;
; MIDIO128
; MIDIO DOUT functions
;
; ==========================================================================
;
; Copyright (C) 1998-2003  Thorsten Klose (Thorsten.Klose@midibox.org)
;                          http://www.uCApps.de
; 
; ==========================================================================
; 
; This file is part of MIDIO128
;
; MIDIO128 is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIDIO128 is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIDIO128; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================

;; --------------------------------------------------------------------------
;;  This function sets a DOUT pin depending on its polarity
;;  IN: pin number in WREG
;; --------------------------------------------------------------------------
MIDIO_DOUT_Set
	SET_BSR	MIDIO_BASE
	BIFSET	MIDIO_CFG0, MIDIO_CFG0_INVERSE_DOUT, BANKED, goto MIOS_DOUT_PinSet0
	goto	MIOS_DOUT_PinSet1

;; --------------------------------------------------------------------------
;;  This function clears a DOUT pin depending on its polarity
;;  IN: pin number in WREG
;; --------------------------------------------------------------------------
MIDIO_DOUT_Clear
	SET_BSR	MIDIO_BASE
	BIFSET	MIDIO_CFG0, MIDIO_CFG0_INVERSE_DOUT, BANKED, goto MIOS_DOUT_PinSet1
	goto	MIOS_DOUT_PinSet0

;; --------------------------------------------------------------------------
;;  This function returns the current status of the DOUT pin depending on
;;  its polarity
;;  IN: pin number in WREG
;; --------------------------------------------------------------------------
MIDIO_DOUT_Get
	call	MIOS_DOUT_PinGet
	SET_BSR	MIDIO_BASE
	BIFSET	MIDIO_CFG0, MIDIO_CFG0_INVERSE_DOUT, BANKED, xorlw 0x01
	return

;; --------------------------------------------------------------------------
;;  This function inverts all DOUT registers - used when the polarity has
;;  been changed
;; --------------------------------------------------------------------------
MIDIO_DOUT_Invert
	;; get pointer to first SR entry
	movlw	0x00
	call	MIOS_DOUT_SRGet

	;; pointer now in FSR1
	;; loop 16 times
	clrf	TMP1
MIDIO_DOUT_InvertLoop
	comf	POSTINC1, F
	incf	TMP1, F
	IFCLR	TMP1, 4, rgoto MIDIO_DOUT_InvertLoop
	
	return


;; --------------------------------------------------------------------------
;;  This function fetches the DIN pins and copies the values into the DOUT
;;  registers if FORWARD_IO function is active.
;; 
;;  It always copies the state into the MIDIO_DIN_VALUES_SRx array
;; 
;;  Has to be requested with MB_STAT, MB_STAT_FETCH_DIN_REQ
;;  Will be called from USER_SR_Service_Finish
;; 
;;  ATTENTION: interrupt service routine --- use IRQ_TMPx instead of TMPx!
;; --------------------------------------------------------------------------
MIDIO_DOUT_FetchDIN
	;; loop 16 times
	clrf	IRQ_TMP1
MIDIO_DOUT_FetchDINLoop
	;; get content of DIN SR
	movf	IRQ_TMP1, W
	call	MIOS_DIN_SRGet
	;; result in WREG and MIOS_PARAMETER1

	;; invert value if INVERSE_DIN set
	SET_BSR	MIDIO_BASE
	btfsc	MIDIO_CFG0, MIDIO_CFG0_INVERSE_DIN, BANKED
	comf	MIOS_PARAMETER1, F
	
	;; copy value to MIDIO_DIN_VALUES_SRx array
	lfsr	FSR1, MIDIO_DIN_VALUES_SR0
	movf	IRQ_TMP1, W
	movff	MIOS_PARAMETER1, PLUSW1

	;; copy value to DOUT if FORWARD_IO is active
	BIFCLR	MIDIO_CFG0, MIDIO_CFG0_FORWARD_IO, BANKED, rgoto MIDIO_DOUT_FetchDINLoopNext

	;; invert value again if INVERSE_DOUT set
	SET_BSR	MIDIO_BASE
	btfsc	MIDIO_CFG0, MIDIO_CFG0_INVERSE_DOUT, BANKED
	comf	MIOS_PARAMETER1, F

	;; write value to DOUT SR
	movf	IRQ_TMP1, W
	call	MIOS_DOUT_SRSet

	;; next
MIDIO_DOUT_FetchDINLoopNext
	incf	IRQ_TMP1, F
	IFCLR	IRQ_TMP1, 4, rgoto MIDIO_DOUT_FetchDINLoop
	return
