;
; MIDIO128
; Customized Meta handler
;
; ==========================================================================
;
; Copyright (C) 1998-2003  Thorsten Klose (Thorsten.Klose@midibox.org)
;                          http://www.uCApps.de
; 
; ==========================================================================
; 
; This file is part of MIDIO128
;
; MIDIO128 is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIDIO128 is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIDIO128; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; ==========================================================================

;; --------------------------------------------------------------------------
;;  This function is called by MIDIO_midi.inc when a meta event (Fx xx) has 
;;  been assigned to a DIN
;;  You can use this handler to enhance the firmware by your own MIDI events.
;;  Here you are able to send more than one MIDI event (i.E. two Note On
;;  Events to control Cakewalk via MIDI remote with one DIN), or a
;;  SysEx/NRPN string to your synthesizer, or just to toggle PIC pins
;;  in order to switch relays...
;;  IN:
;;     on DIN events (entry point: MIDIO_META_Handler_DIN):
;;        o Pot number in MIDIO_CURRENT_DIN (BANKED access required!)
;;        o first MIDI byte in MIDI_EVNT0 (no BANKED access required)
;;        o second MIDI byte in MIDI_EVNT1 (no BANKED access required)
;;        o DIN value in MIDI_EVNT_VALUE (no BANKED access required)
;; --------------------------------------------------------------------------
MIDIO_META_Handler_DIN

	;; THIS IS JUST AN EXAMPLE META EVENT HANDLER
	;; ADAPT IT FOR YOUR NEEDS!

	;; we are using the same handler for dins here
	rgoto	MIDIO_META_Handler


;; --------------------------------------------------------------------------
;; THIS IS JUST AN EXAMPLE META EVENT HANDLER
;; ADAPT IT FOR YOUR NEEDS!
;; HINT: if a large number of SysEx strings with the same structure (means:
;; for the same synth) should be sent, it would be more advantageous to
;; use a table which contains the parameter values and refers to the
;; sending routines
;; --------------------------------------------------------------------------
MIDIO_META_Handler
	;; don't send anything if meta event is 0xff
	movf	MIDI_EVNT0, W
	xorlw	0xff
	skpnz
	return

	;; branch depending on second meta event byte
	movf	MIDI_EVNT1, W
	JUMPTABLE_2BYTES 17	; 17 entries
	rgoto	MIDIO_META_Handler_00
	rgoto	MIDIO_META_Handler_01
	rgoto	MIDIO_META_Handler_02
	rgoto	MIDIO_META_Handler_03
	rgoto	MIDIO_META_Handler_04
	rgoto	MIDIO_META_Handler_05
	rgoto	MIDIO_META_Handler_06
	rgoto	MIDIO_META_Handler_07
	rgoto	MIDIO_META_Handler_08
	rgoto	MIDIO_META_Handler_09
	rgoto	MIDIO_META_Handler_0A
	rgoto	MIDIO_META_Handler_0B
	rgoto	MIDIO_META_Handler_0C
	rgoto	MIDIO_META_Handler_0D
	rgoto	MIDIO_META_Handler_0E
	rgoto	MIDIO_META_Handler_0F
	rgoto	MIDIO_META_Handler_10

;; --------------------------------------------------------------------------
MIDIO_META_Handler_00
	;; this example sends a SysEx string: F0 43 10 5C 10 10 31 vv F7 (AN1x: high pass filter)
	;; META Event: "F0 00" (pot) or "F0 00 7F @OnOff" (Button - 7F may vary)
	movlw	0x31			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_01
	;; this example sends a SysEx string: F0 43 10 5C 10 10 33 vv F7 (AN1x: cutoff frequency)
	;; META Event: "F0 01" (pot) or "F0 01 7F @OnOff" (Button - 7F may vary)	
	movlw	0x33			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_02
	;; this example sends a SysEx string: F0 43 10 5C 10 10 34 vv F7 (AN1x: resonance)
	;; META Event: "F0 02" (pot) or "F0 02 7F @OnOff" (Button - 7F may vary)	
	movlw	0x34			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_03
	;; this example sends a SysEx string: F0 43 10 5C 10 10 29 vv F7 (AN1x: volume VCO1)
	;; META Event: "F0 03" (pot) or "F0 03 7F @OnOff" (Button - 7F may vary)	
	movlw	0x29			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_04
	;; this example sends a SysEx string: F0 43 10 5C 10 10 2A vv F7 (AN1x: volume VCO2)
	;; META Event: "F0 04" (pot) or "F0 04 7F @OnOff" (Button - 7F may vary)	
	movlw	0x2a			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_05
	;; this example sends a SysEx string: F0 43 10 5C 10 10 2C vv F7 (AN1x: volume noise)
	;; META Event: "F0 05" (pot) or "F0 05 7F @OnOff" (Button - 7F may vary)	
	movlw	0x2c			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_06
	;; this example sends a SysEx string: F0 43 10 5C 10 10 3E vv F7 (AN1x: feedback)
	;; META Event: "F0 06" (pot) or "F0 06 7F @OnOff" (Button - 7F may vary)	
	movlw	0x3e			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
MIDIO_META_Handler_07
	;; this example sends a SysEx string: F0 43 10 5C 10 10 3F vv F7 (AN1x: volume)
	;; META Event: "F0 07" (pot) or "F0 07 7F @OnOff" (Button - 7F may vary)	
	movlw	0x3f			; store parameter value in W
	rgoto	META_Send_AN1x_SysEx	; branch to SysEx routine

;; --------------------------------------------------------------------------
	;; This is a subroutine which sends a SysEx stream for AN1x synthesizer
	;; IN: parameter address in working register WREG
	;;     pot/button value in MIDI_SEND
META_Send_AN1x_SysEx
	movwf	TMP1		; temporary store parameter value from working register W into TMP1
	call	MIOS_MIDI_BeginStream	; begin stream
	movlw	0xf0			; send 0xf0
	call	MIOS_MIDI_TxBufferPut
	movlw	0x43			; send 0x43
	call	MIOS_MIDI_TxBufferPut
	movlw	0x10			; send 0x10
	call	MIOS_MIDI_TxBufferPut
	movlw	0x5c			; send 0x5c
	call	MIOS_MIDI_TxBufferPut
	movlw	0x10			; send 0x10
	call	MIOS_MIDI_TxBufferPut
	movlw	0x10			; send 0x10
	call	MIOS_MIDI_TxBufferPut
	movf	TMP1, W			; send parameter value, stored in TMP1
	call	MIOS_MIDI_TxBufferPut
	movf	MIDI_EVNT_VALUE, W	; send pot/button value, stored in MIDI_EVNT_VALUE
	call	MIOS_MIDI_TxBufferPut
	movlw	0xf7			; send 0xf7
	call	MIOS_MIDI_TxBufferPut
	goto	MIOS_MIDI_BeginStream	; end stream and exit

;; --------------------------------------------------------------------------

	;; a trick: all labels are pointing to the same routine
MIDIO_META_Handler_08
MIDIO_META_Handler_09
MIDIO_META_Handler_0A
MIDIO_META_Handler_0B
MIDIO_META_Handler_0C
MIDIO_META_Handler_0D
MIDIO_META_Handler_0E
MIDIO_META_Handler_0F

	;; this example sends two NOTE On or Off Events in order to control the
	;; MIDI Remote Function of Cakewalk (or Cubase or Logic)
	;; META Event: "F0 0x" (pot) or "F0 0x 36 @OnOff" (Button - 36 may vary and is the second MIDI note)
	;; x = 08 to 0F

MIDIO_META_Handler_DoubleNote
	call	MIOS_MIDI_BeginStream	; begin stream

	;; branch depending on content of MIDI_EVNT_VALUE
	movf	MIDI_EVNT_VALUE, W		; get MIDI_EVNT_VALUE value
	bz	MIDIO_META_Handler_DoubleNoteOff	; when Zero (pot value = 00 or button released): send Note Off

MIDIO_META_Handler_DoubleNoteOn
	;; NOTE ON
	movlw	0x90		; send 0x90 (Note On Header, channel 0)
	call	MIOS_MIDI_TxBufferPut
	movlw	0x00		; send 0x00 (C-0) -- cakewalk needs this as MIDI Remote indicator
	call	MIOS_MIDI_TxBufferPut
	movlw	0x7f		; send 0x7F (velocity)
	call	MIOS_MIDI_TxBufferPut

	movf	MIDI_EVNT1, W	; send content of second byte (08-0f)
	call	MIOS_MIDI_TxBufferPut
	movlw	0x7f		; send 0x7F (velocity)
	call	MIOS_MIDI_TxBufferPut

	rgoto	MIDIO_META_Handler_DoubleNoteEnd

	;; ----
	
MIDIO_META_Handler_DoubleNoteOff
	;; NOTE OFF
	movlw	0x80		; send 0x80 (Note Off Header, channel 0)
	call	MIOS_MIDI_TxBufferPut
	movlw	0x00		; send 0x00 (C-0) -- cakewalk needs this as MIDI Remote indicator
	call	MIOS_MIDI_TxBufferPut
	movlw	0x00		; send 0x00 (velocity) -- 0x00 = NOTE OFF
	call	MIOS_MIDI_TxBufferPut
	movf	MIDI_EVNT1, W	; send content of second byte (08-0f)
	call	MIOS_MIDI_TxBufferPut
	movlw	0x00		; send 0x00 (velocity) -- 0x00 = NOTE OFF
	call	MIOS_MIDI_TxBufferPut

MIDIO_META_Handler_DoubleNoteEnd
	goto	MIOS_MIDI_BeginStream	; end stream and exit


	;; ------------------------------------------------------------------
MIDIO_META_Handler_10
	;; this example can be used to send a single-byte System Realtime event with a button:
	;;    F8 00 [00-7F]: send MIDI Clock
	;;    FA 00 [00-7F]: send MIDI Start
	;;    FB 00 [00-7F]: send MIDI Continue
	;;    FC 00 [00-7F]: send MIDI Stop
	movf	MIDI_EVNT0, W		; extract status from first byte
	andlw	0x0f
	iorlw	0xf0			; add SysEx status
	goto	MIOS_MIDI_TxBufferPut	; send byte and exit

